
# Operating System Intergrity Checker

The Operating System Integrity Checker is a tool used to verify that your
installed files were not tempered with.

The tool reads files on this system, computes their SHA512 and gather the
`stat()` information. This is then sent to another computer for verification.
The verification happens on that other computer, not on the computer being
checked. This means more security since a hacker should not be able to
connect to the other computer and pretend that files are valid.


# Basic Client Algorithm

A client with the `os-integrity-checker` service connects to the remote
server and works with that server to proceed with its checks.

1. Initialization

   Just after installation, the first step for the basic server is to check
   all the existing files found on the server and send the information to
   the remote controller with the SHA512 and `stat()` data.

2. Processing

   Once the initialization is over, the client enters the processing phase.
   This means it checks each file and sends the data to the remote computer.
   If there is an issue, the remote computer will react accordingly.


# Basic Server Algorithm

On the server, the algorithm ensures that all the files of a clients are
being checked on a daily basis.

The client sends information about each file. The server marks the files as
checked on this date and verifies that the data is as expected. If the SHA512
or the `stat()` values are invalid, then the server sends an email to the
administrator (the email is a digest for the day, week, month...)

The server has a website interface showing the current status of each client.


# Implementation

## Client/Server Connection

The client and server directly connect between each others using a messenger.
This uses a direct connection, opposed to using the communicator daemon, to
avoid potential security issues with said daemon.

The connection is always secure (there is no option to connect over a plain
text channel). If an encrypted connection is not possible, then the connection
fails.

## Client Starts -- `CONNECT`

When the client comes live, the first thing it does is send a `CONNECT`
message:

    CONNECT name=<client's name>;ip=<client's ip>;...

Parameters:

* `name` -- name of the client; this is the hostname
* `ip` -- the IP address of the client; if you expect the client to not
  change then this is going to be static and we can verify that
* `bootid` -- the ID of the last boot so we know whether the system was
  restarted

Once the server receives the `CONNECT` message, the rest of the communication
is 100% controlled by the server. The client becomes the server's extension
to check the files on the client's machine. In other words, it becomes a state
machine and the server controls the state.

**Note:** The server can be setup to only accept known clients. This
          works well with a static installation--an installation
          where no new computers get spawned automatically. With this
          setup, the administrator is responsible for adding new clients
          on the server otherwise the server will refuse those connections.


## Installed Packages -- `LIST_PACKAGES` / `PACKAGES`

The first time the client connects, it is expected that the client is in a
_safe state_ (not yet hacked). At that point, the client can send all its
existing data in order to define its current state.

On future connections, the list of packages is also queried but if it
changes then the changes are reported to the administrator.

When the server wants the list of packages found on a client's machine it
sends the following message:

    LIST_PACKAGES

In turn, the client replies with a list of all the packages installed
locally:

    PACKAGES list=<package 1>|<version 1>,<package 2>|<version 2>,...

Parameter:

* `list` -- the list of packages name and version separated by a '|'

The client generates the list of packages using the `dpkg-query` command
like so:

    dpkg-query --show --showformat='${Package}|${Version},'

When the server receives the `PACKAGES` message, it saves the information in
a file which it uses to query that client. It determines the number of files
and schedule the next messages to ask for the current state of each file in
a way which allows for all the files to be checked in about 24 hours.


## Configuration Files -- `LIST_FILES` / `FILES`

The server also requests the list of configuration files under `/etc`.
Like packages, configuration files may be modified, added or removed and
all of that generates events.

Note that, in most cases, you limit this list to configuration files. As the
administrator, you may also add other files that you deem useful in your
check.

Later the server requests the SHA512 of the files that still exist or of
new files.

    LIST_FILES dirs=<dir 1>,<dir 2>,...,start_page=1

Parameter:

* `dirs` -- A list of comma separated directory names, with inclusion and
  exclusion patterns (see syntax below).
* `start_page` -- in most cases, this parameter is set to 1. If a previous
  `LIST_FILES` failed midstream, this parameter is used to avoid
  retransmitting all the files that were already received by the server.
* `include` -- a list of global inclusive patterns.
* `exclude` -- a list of global exclusive patterns.

The client answers that message with:

    FILES page=<pos>,final=<true|false>,list=<file 1>,<file 2>,...

Parameters:

* `page` -- This message can be paginated if the complete list is larger
than 16Kb. This is the page number starting at 1.
* `final` -- Whether this page is the final page. This indicates to the
server that the list is now complete.
* `list` -- the list of filenames as returned by a glob().

To minimize the impact on the network, the client answers with messages that
are around 16Kb or less (the content of the `list=...` parameter; the total
message size may still be larger).

### Directory / Inclusion / Exclusion Format

Each directory specified in the `LIST_FILES` messages may include
inclusion and exclusion patterns. These patterns are used to allow or
disallow files in the list.

The format is the name of the directory followed by inclusion patterns
written between square brackets (`[*.conf]`) and exclusion patterns
written between parenthesis (`(*.bak]`):

    start: segment_list inclusion_list
    segment_list: segment
                | segment_list '/' segment
    inclusion_list: inclusion
                  | inclusion_list inclusion
    inclusion: '[' pattern ']'
    exclusion_list: exclusion
                  | exclusion_list exclusion
    exclusion: '(' exclusion ')'
    segment: [^/[(*?]+
    pattern: segment
           | [*?]
           | pattern pattern

The pattern does not require the inclusion of the `*` or `?` wildcards.
If, for example, you have a directory `/etc/apache2/certificates` and you
do not want to have those checked (possibly because they change all the
time), then you can make use of the pattern `certificates` and that
directory and any file under it will not be included:

    /etc(certificates)


## Verify a File -- `VERIFY` / `STATS`

Finally, the server requests the statistics of one or more files with the
`VERIFY` message.

    VERIFY files=<name 1>,<name 2>,...

Parameter:

* `files` -- the list of files to check and return the statistics about.

The client is expected to answer with the following message:

    STATS file=<name>;sha512=<hash>;murmur3=<hash>;mode=<r/w/x flags, type>;
          dev=<device ID>;ino=<inode>;links=<nlink>;uid=<uid>;gid=<gid>;
          size=<file size>;blocks=<block count>;rdev=<special file>;
          mtime=<modification time>;ctime=<change time>

Parameters:

* `file` -- the full path and filename of the file that was checked; the
  other parameters are specific to this file.
* `exists` -- if this parameter is set to false, we could not find the file
* `sha512` -- the SHA512 of the file contents.
* `murmur3` -- the Murmur3 of the file contents.
* `mode` -- the file mode (rwxrwxrwx) and file type.
* `dev` -- the device on which the file resides, it should not change.
* `ino` -- the inode number where the file starts.
* `links` -- number of hard links.
* `uid` -- the user identifier (integer).
* `gid` -- the group identifier (integer).
* `size` -- the file size in bytes.
* `blocks` -- the number of blocks used by the file.
* `rdev` -- the special file device identifier.
* `mtime` -- last modification time.
* `ctime` -- last change time.

### Special File Handling

For files that are not regular files (i.e. other than a text file, an
executable, a library), the client process only checks the statistics and
whether the file exists or is missing. It does not open the file and compute
the SHA512 and Murmur3 hashes.

### Package Verification

For packages, the server downloads its own copy of the `.deb` file and uses
its copy to verify the files' SHA512 & Murmur3 as required. In other words,
we do not trust the client beyond it sending us the correct information.
Anything else and we report an issue. For security reasons, the client does
not keep a copy of the data. Once sent, it is discarded.

**Why two hashes?** Because the SHA512 and Murmur3 use completely different
algorithms, their results are nearly always different. It makes it really
difficult to change the file as a result. Already with just one hash, it
is complicated to get the hash to match after you made changes, but with
two it's nearly impossible.

### Configuration Files Check

For configuration files, the server uses the information that was sent to
it from the client when it was first initialized. Many configuration files
are the same as the ones found in packages. Those are checked against the
package file, but the server knows an administrator may change those files.
When it changes, the new information gets saved on the server and the event
is recorded along.

### Constant Check

The verification process occurs throughout the day. So if you had exactly
86,400 files to check, the system would verify one file per second on average.

Note that in reality the client needs to verify a total number of bytes per
seconds instead of a certain number of files because some files are really big
and will take much longer than others that are close to empty and many very
small files can very quickly be checked together.


## File Database

The OS Integrity Checker manages one database per client. The database is
split in several files as defined below.

### Location

The database files are saved under:

    /var/lib/sitter/integrity/<uuid>/...

### Client Information (client.conf)

The client data is a simple configuration file. It includes the following
fields:

* `name` -- descriptive name of the client
* `client` -- the IP address or domain name to access the client
* `type` -- "front-end", "service", "backend", "database", "backup"
* `importance` -- "ignore", "deprecated", "low", "medium", "high", "secure"

At this point, the only important parameter is the `client` field which is
used to verify that the client is indeed one we expect.

### Sequential Package List (packages.db)

The server asks the client about the list of installed packages. It uses
that information to determine the list of files expected on the destination
system.

* Package ID (assigned sequential number starting at 1)
* Package Name

### Sequential File List (files.db)

This file is sequential. It uses exactly N bytes per file entry. If the
filename cannot fit in the default field, then it gets added to the
Filenames Repository file instead.

* File ID
* Filename (see String Repository)
* From Package (Package ID), if 0, this is not from a package (i.e. .conf)
* File Source/Importance:
  * Package (read-only, just cannot change)
  * Fixed Configuration (read-only, not expected to change)
  * Editable Configuration (expected to be edited by administrators)
  * Log (hash/size/mtime ignored, UID/GID/mode not expected to change)
  * Other

### State Repository (state.db)

This file records the current state of each file for a client.

* File ID
* File Stats (see STATS message above)

### String Repository (strings.db)

This file is used to save strings that do not otherwise fit in the default
structures. This allows us to have sequential files for all the other
structures which allow us to easily access any entry with a simple `seek()`.

The management of this file is more complex since strings cannot be moved
without also updating the offset found in the corresponding sequential files.

A filename entry includes these elements:

* Which file is referencing this string (1 byte)
* Index of file in Sequential File List (4 bytes)
* Size of the string (2 bytes)
* The string

One of the processes running against this table is a compression process which
moves strings at a lower offset within the file. This happens when a strings
gets deleted in order to reclaim the now unused space. We also can directly
reuse the space used by a string but it is not always easy to match the size
of an old string.

The first byte has two special values:

* `0x00` -- there is no string here
* `0xFF` -- the string was deleted and it can be removed

Deleted strings make use of the index 4 bytes to link to the next deleted
string of the same size. The first of those strings appears in the header.


## Sample from Tripwire

The following are the fields shown by Tripwire when a file changed:

    Modified object name:  /usr/lib/mysql/plugin/adt_null.so

      Property:            Expected                    Observed
      -------------        -----------                 -----------
      Object Type          Regular File                Regular File
      Device Number        2067                        2067
    * Inode Number         39584856                    39584079
      Mode                 -rw-r--r--                  -rw-r--r--
      Num Links            1                           1
      UID                  root (0)                    root (0)
      GID                  root (0)                    root (0)
      Size                 21224                       21224
    * Modify Time          Tue Jul 26 04:28:55 2022    Wed Oct 19 05:58:42 2022
      Blocks               48                          48
    * CRC32                AcoYuR                      Dpkm/x
    * MD5                  CZkPPydrdtfCKC9rAiGAly      D2HWu/nQVOARjMS4hRmHHB



# vim: ts=4 sw=4 et
